from unittest import skipIf

from django.test import TestCase

from faker import Faker

from drc_cmis.client import default_client
from drc_cmis.exceptions import DocumentConflictException
from drc_cmis.storage import BinaireInhoud

from ..factories import (
    DRCCMISConnectionFactory, EnkelvoudigInformatieObjectFactory
)
from ..mixins import DMSMixin

fake = Faker()


class CMISClientTests(DMSMixin, TestCase):
    def test_update_zaakdocument_only_props(self):
        document = EnkelvoudigInformatieObjectFactory.create()

        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        default_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)

        # Update the document
        new_file_name = fake.file_name()
        document.titel = new_file_name
        document.beschrijving = 'Andere beschrijving'
        document.save()

        result = self.cmis_client.update_zaakdocument(koppeling)
        self.assertIsNone(result)

        cmis_doc = self.cmis_client._get_cmis_doc(document)
        cmis_doc = cmis_doc.getLatestVersion()
        self.assertExpectedProps(
            cmis_doc, {
                'cmis:contentStreamLength': document.inhoud.size,
                'zsdms:documentIdentificatie': str(document.identificatie),
                'cmis:versionSeriesCheckedOutId': None,
                'cmis:name': new_file_name,
                'zsdms:documentbeschrijving': 'Andere beschrijving',
            }
        )

    def test_update_zaakdocument_content(self):
        zaak_url = 'http://zaak.nl/locatie'
        self.cmis_client.creeer_zaakfolder(zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)

        new_file_name = fake.file_name()
        inhoud = BinaireInhoud(b'leaky abstraction...', filename=new_file_name)

        result = self.cmis_client.update_zaakdocument(koppeling, inhoud=inhoud)
        self.assertIsNone(result)

        filename, content = self.cmis_client.geef_inhoud(document)
        self.assertEqual(filename, new_file_name)
        self.assertEqual(content.read(), b'leaky abstraction...')

        cmis_doc = self.cmis_client._get_cmis_doc(document)
        cmis_doc = cmis_doc.getLatestVersion()
        self.assertExpectedProps(
            cmis_doc, {
                'cmis:contentStreamLength': 20,
                'zsdms:documentIdentificatie': str(document.identificatie),
                'cmis:versionSeriesCheckedOutId': None,
                'cmis:name': new_file_name,
            }
        )

    def test_update_checked_out_zaakdocument(self):
        zaak_url = 'http://zaak.nl/locatie'
        self.cmis_client.creeer_zaakfolder(zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)
        cmis_doc = self.cmis_client._get_cmis_doc(document)
        cmis_doc.checkout()

        new_file_name = fake.file_name()
        inhoud = BinaireInhoud(b'leaky abstraction...', filename=new_file_name)

        with self.assertRaises(DocumentConflictException):
            self.cmis_client.update_zaakdocument(koppeling, inhoud=inhoud)

    def test_update_checked_out_zaakdocument_with_checkout_id(self):
        zaak_url = 'http://zaak.nl/locatie'
        self.cmis_client.creeer_zaakfolder(zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        cmis_doc = self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)

        result = self.cmis_client.is_locked(document)
        self.assertFalse(result)

        pwc = cmis_doc.checkout()
        pwc.reload()
        checkout_id = pwc.properties['cmis:versionSeriesCheckedOutId']

        # result = self.cmis_client.is_locked(document)
        # self.assertFalse(result)

        new_file_name = fake.file_name()
        inhoud = BinaireInhoud(b'leaky abstraction...', filename=new_file_name)

        result = self.cmis_client.update_zaakdocument(koppeling, checkout_id=checkout_id, inhoud=inhoud)
        self.assertIsNone(result)

        filename, content = self.cmis_client.geef_inhoud(document)
        self.assertEqual(filename, new_file_name)
        self.assertEqual(content.read(), b'leaky abstraction...')

        # check that it's checked in again
        new_pwc = cmis_doc.getPrivateWorkingCopy()
        self.assertIsNone(new_pwc)

    def test_update_checked_out_zaakdocument_with_incorrect_checkout_id(self):
        zaak_url = 'http://zaak.nl/locatie'
        self.cmis_client.creeer_zaakfolder(zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)
        cmis_doc = self.cmis_client._get_cmis_doc(document)
        cmis_doc.checkout()

        new_file_name = fake.file_name()
        inhoud = BinaireInhoud(b'leaky abstraction...', filename=new_file_name)

        with self.assertRaises(DocumentConflictException):
            self.cmis_client.update_zaakdocument(koppeling, checkout_id='definitely not right', inhoud=inhoud)



from time import time
from unittest import skipIf

from django.test import TestCase

from cmislib.exceptions import ObjectNotFoundException

from ..mixins import DMSMixin


class CMISClientTests(DMSMixin, TestCase):
    def test_boomstructuur(self):
        """
        Test dat de boomstructuur Zaken -> Zaaktype -> Zaak gemaakt wordt.
        """
        with self.assertRaises(ObjectNotFoundException):
            self.cmis_client._repo.getObjectByPath('/Zaken')

        self.cmis_client.creeer_zaakfolder(self.zaak_url)

        # Zaken root folder
        root_folder = self.cmis_client._repo.getObjectByPath('/Zaken')

        children = [child for child in root_folder.getChildren()]
        self.assertEqual(len(children), 1)

        # zaak subfolder
        zaak_folder = children[0]
        self.assertEqual(zaak_folder.name, 'httpzaaknllocatie')
        self.assertExpectedProps(zaak_folder, {
            'cmis:objectTypeId': 'F:zsdms:zaak',
            'cmis:baseTypeId': 'cmis:folder',
            'cmis:path': '/Zaken/httpzaaknllocatie',
            'zsdms:startdatum': None,
            'zsdms:einddatum': None,
            'zsdms:zaakniveau': None,  # TODO
            'zsdms:deelzakenindicatie': None,  # TODO
            'zsdms:registratiedatum': None,
            'zsdms:archiefnominatie': None,
            'zsdms:datumVernietigDossier': None,
        })

    def test_boomstructuur_unique_name(self):
        """
        Test dat de boomstructuur Zaken -> Zaaktype -> Zaak gemaakt wordt.
        """
        with self.assertRaises(ObjectNotFoundException):
            self.cmis_client._repo.getObjectByPath('/Zaken')

        stamp = time()
        self.cmis_client.creeer_zaakfolder(stamp)

        # Zaken root folder
        root_folder = self.cmis_client._repo.getObjectByPath('/Zaken')

        children = [child for child in root_folder.getChildren()]
        self.assertEqual(len(children), 1)

        # zaak subfolder
        zaak_folder = children[0]
        self.assertEqual(zaak_folder.name, '{}'.format(stamp).replace('.', ''))
        self.assertExpectedProps(zaak_folder, {
            'cmis:objectTypeId': 'F:zsdms:zaak',
            'cmis:baseTypeId': 'cmis:folder',
            'cmis:path': '/Zaken/{}'.format(stamp).replace('.', ''),
            'zsdms:startdatum': None,
            'zsdms:einddatum': None,
            'zsdms:zaakniveau': None,  # TODO
            'zsdms:deelzakenindicatie': None,  # TODO
            'zsdms:registratiedatum': None,
            'zsdms:archiefnominatie': None,
            'zsdms:datumVernietigDossier': None,
        })



# Generated by Django 2.2 on 2019-04-19 08:29

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EnkelvoudigInformatieObject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, unique=True)),
                ('identificatie', models.CharField(default=uuid.uuid4, max_length=40)),
                ('bronorganisatie', models.CharField(max_length=9)),
                ('creatiedatum', models.DateField()),
                ('titel', models.CharField(max_length=200)),
                ('vertrouwelijkheidaanduiding', models.CharField(max_length=200)),
                ('auteur', models.CharField(max_length=200)),
                ('status', models.CharField(max_length=20)),
                ('beschrijving', models.TextField(max_length=1000)),
                ('ontvangstdatum', models.DateField(blank=True, null=True)),
                ('verzenddatum', models.DateField(blank=True, null=True)),
                ('indicatie_gebruiksrecht', models.NullBooleanField(default=None)),
                ('ondertekening_soort', models.CharField(blank=True, max_length=10)),
                ('ondertekening_datum', models.DateField(blank=True, null=True)),
                ('informatieobjecttype', models.URLField()),
                ('formaat', models.CharField(blank=True, max_length=255)),
                ('taal', models.CharField(blank=True, max_length=255)),
                ('bestandsnaam', models.CharField(blank=True, max_length=255)),
                ('inhoud', models.FileField(upload_to='uploads/%Y/%m/')),
                ('link', models.URLField(blank=True)),
                ('integriteit_algoritme', models.CharField(blank=True, max_length=20)),
                ('integriteit_waarde', models.CharField(blank=True, max_length=128)),
                ('integriteit_datum', models.DateField(blank=True, null=True)),
            ],
        ),
    ]



from django.contrib import admin

from .models import DRCCMISConnection


@admin.register(DRCCMISConnection)
class DRCCMISConnectionAdmin(admin.ModelAdmin):
    list_display = ('enkelvoudiginformatieobject', )
    search_fields = ('enkelvoudiginformatieobject', )



from django.db.models import signals
from django.urls import reverse

import factory
from django_webtest import WebTest

from drc_cmis.client import default_client

from .factories import (
    DRCCMISConnectionFactory, EnkelvoudigInformatieObjectFactory
)
from .mixins import DMSMixin


class DownloadFileViewTests(DMSMixin, WebTest):
    @factory.django.mute_signals(signals.post_save)
    def test_download_file_404(self):
        enkelvoudig_informatie_object = EnkelvoudigInformatieObjectFactory()
        url = reverse('cmis:cmis-document-download', kwargs={'inhoud': enkelvoudig_informatie_object.uuid})
        self.app.get(url, status=404)

    def test_download_file(self):
        enkelvoudig_informatie_object = EnkelvoudigInformatieObjectFactory()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=enkelvoudig_informatie_object)
        default_client.maak_zaakdocument_met_inhoud(koppeling, stream=enkelvoudig_informatie_object.inhoud)
        enkelvoudig_informatie_object.inhoud.seek(0)
        url = reverse('cmis:cmis-document-download', kwargs={'inhoud': enkelvoudig_informatie_object.uuid})
        response = self.app.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.body, enkelvoudig_informatie_object.inhoud.read())



from io import BytesIO

from django.test import TestCase

from drc_cmis.storage import BinaireInhoud


class BinaureInhoudTests(TestCase):
    def test_to_cmis(self):
        inhoud = BinaireInhoud(None, 'filename')
        self.assertEqual(inhoud.to_cmis(), None)

    def test_to_cmis_with_inhoud(self):
        inhoud = BinaireInhoud(b'test', 'filename')
        self.assertEqual(inhoud.to_cmis().read(), b'test')



from django.apps import apps
from django.http.response import Http404, HttpResponse
from django.utils.encoding import smart_str
from django.views.generic import DetailView

from drc_cmis import settings

from .client import default_client


class DownloadFileView(DetailView):
    model = apps.get_model(*settings.DRC_CMIS_ENKELVOUDIGINFORMATIEOBJECT.split("."))
    slug_field = "uuid"
    slug_url_kwarg = "inhoud"

    def get(self, request, *args, **kwargs):
        document = self.get_object()
        filename, content = default_client.geef_inhoud(document)

        file_content = content.read()

        if file_content:
            response = HttpResponse(file_content, content_type="application/force-download")
            response["Content-Disposition"] = "attachment; filename=%s.bin" % smart_str(filename)
            return response
        raise Http404



from unittest import skipIf

from django.test import TestCase

from drc_cmis.abstract import DRCClient


class CMISClientTests(TestCase):
    def setUp(self):
        self.client = DRCClient()

    def test_creeer_zaakfolder(self):
        with self.assertRaises(NotImplementedError):
            self.client.creeer_zaakfolder('zaak_url')

    def test_maak_zaakdocument(self):
        with self.assertRaises(NotImplementedError):
            self.client.maak_zaakdocument('document')

    def test_maak_zaakdocument_met_inhoud(self):
        with self.assertRaises(NotImplementedError):
            self.client.maak_zaakdocument_met_inhoud('document')

    def test_geef_inhoud(self):
        with self.assertRaises(NotImplementedError):
            self.client.geef_inhoud('document')

    def test_zet_inhoud(self):
        with self.assertRaises(NotImplementedError):
            self.client.zet_inhoud('document', 'stream')

    def test_relateer_aan_zaak(self):
        with self.assertRaises(NotImplementedError):
            self.client.relateer_aan_zaak('document', 'zaak_url')

    def test_update_zaakdocument(self):
        with self.assertRaises(NotImplementedError):
            self.client.update_zaakdocument('document')

    def test_checkout(self):
        with self.assertRaises(NotImplementedError):
            self.client.checkout('document')

    def test_cancel_checkout(self):
        with self.assertRaises(NotImplementedError):
            self.client.cancel_checkout('document', 'checkout_id')

    def test_ontkoppel_zaakdocument(self):
        with self.assertRaises(NotImplementedError):
            self.client.ontkoppel_zaakdocument('document', 'zaak_url')

    def test_is_locked(self):
        with self.assertRaises(NotImplementedError):
            self.client.is_locked('document')

    def test_verwijder_document(self):
        with self.assertRaises(NotImplementedError):
            self.client.verwijder_document('document')

    def test_sync(self):
        with self.assertRaises(NotImplementedError):
            self.client.sync()



from django.contrib.admin.utils import get_fields_from_path
from django.db.models.constants import LOOKUP_SEP

from .choices import CMISObjectType


def get_cmis_object_id_parts(cmis_object_id):
    """
    Returns the actual object ID, version and path, if present.

    :param object_id: A `CmisId`.
    :return: A `tuple` containing the actual object Id, version and path as strings.
    """
    parts = cmis_object_id.split(";")
    version = None
    if len(parts) == 2:
        version = parts[1]
    parts = parts[0].rsplit("/")
    object_id = parts[-1]
    path = None
    if len(parts) == 2:
        path = parts[0]
    return (object_id, version, path)


def get_cmis_object_id(cmis_object_id):
    """
    Returns the actual object ID.

    :param object_id: A `CmisId`.
    :return: The actual CMIS Id as string.
    """
    return get_cmis_object_id_parts(cmis_object_id)[0]


class FolderConfig:
    __slots__ = ["type", "name"]

    def __init__(self, type_=None, name=None):
        if not type_ and not name:
            raise AssertionError("Either type or name is required")
        self.type = type_
        self.name = name

    def __repr__(self):
        return ("<{} type_={!r} name={!r}>").format(self.__class__.__name__, self.type, self.name)


def upload_to(zaak_url):
    """
    Return the fully qualified upload path for the zaak, generic case.

    Each item from the return list is a FolderConfig object with either a
    type, name or both defined. If a name is defined, this name will be used
    for the folder name. The type is required to be able to generate the
    appropriate cmis properties.

    :param zaak_url: :class:`string`.
    :return: list of FolderConfig objects, in order of root -> leaf
    """
    return [FolderConfig(name="Zaken", type_=CMISObjectType.zaken), FolderConfig(type_=CMISObjectType.zaak_folder)]


def get_model_value(obj, field_name):
    """
    Returns the value belonging to `field_name` on `Model` instance.
    This works for related fields.

    Example::

        >>> get_model_value(Zaak, 'zaaktype__zaaktypeomschrijving')
        'Some description'

    """
    fields = field_name.split(LOOKUP_SEP)
    for field in fields:
        obj = getattr(obj, field)

    return obj


def get_model_field(model, field_name):
    """
    Returns the `Field` instance belonging to `field_name` on a `Model`
    instance or class. This works for related fields.

    Example::

        >>> get_model_field(Zaak, 'zaaktype__zaaktypeomschrijving')
        <django.db.models.fields.CharField: zaaktypeomschrijving>

    """
    return get_fields_from_path(model, field_name)[-1]



from time import time
from unittest import skipIf

from django.test import TestCase

from cmislib.exceptions import ObjectNotFoundException

from drc_cmis.choices import CMISObjectType
from drc_cmis.utils import FolderConfig

from ..mixins import DMSMixin


class CMISClientTests(DMSMixin, TestCase):
    def test_get_or_create_folder(self):
        """
        """
        with self.assertRaises(ObjectNotFoundException):
            self.cmis_client._repo.getObjectByPath('/Zaken')

        stamp = '{}'.format(time()).replace('.', '')
        self.cmis_client.creeer_zaakfolder(stamp)

        # Zaken root folder
        root_folder = self.cmis_client._repo.getObjectByPath('/Zaken')

        children = [child for child in root_folder.getChildren()]
        self.assertEqual(len(children), 1)

        self.cmis_client.creeer_zaakfolder(stamp)
        children = [child for child in root_folder.getChildren()]
        self.assertEqual(len(children), 1)

        self.cmis_client.creeer_zaakfolder('other')
        children = [child for child in root_folder.getChildren()]
        self.assertEqual(len(children), 2)

    def test_get_folder_name_with_name(self):
        with self.assertRaises(ObjectNotFoundException):
            self.cmis_client._repo.getObjectByPath('/Zaken')

        stamp = '{}'.format(time()).replace('.', '')
        self.cmis_client.creeer_zaakfolder(stamp)

        name = self.cmis_client.get_folder_name(stamp, FolderConfig(type_=CMISObjectType.zaken, name='test'))
        self.assertEqual(name, 'test')

    def test_get_folder_name_zaak_folder(self):
        with self.assertRaises(ObjectNotFoundException):
            self.cmis_client._repo.getObjectByPath('/Zaken')

        stamp = '{}'.format(time()).replace('.', '')
        self.cmis_client.creeer_zaakfolder(stamp)

        name = self.cmis_client.get_folder_name(stamp, FolderConfig(type_=CMISObjectType.zaak_folder))
        self.assertEqual(name, stamp)

    def test_get_folder_name_no_name(self):
        with self.assertRaises(ObjectNotFoundException):
            self.cmis_client._repo.getObjectByPath('/Zaken')

        stamp = '{}'.format(time()).replace('.', '')
        self.cmis_client.creeer_zaakfolder(stamp)

        with self.assertRaises(ValueError):
            self.cmis_client.get_folder_name(stamp, FolderConfig(type_=CMISObjectType.zaken))



# TODO, Test the sync function of the client
from collections import OrderedDict
from datetime import date, datetime
from io import BytesIO
from unittest import skipIf

from django.test import TestCase, override_settings

import pytz

from drc_cmis.exceptions import DocumentExistsError, SyncException
from drc_cmis.models import ChangeLog

from ..factories import EnkelvoudigInformatieObjectFactory
from ..mixins import DMSMixin


class CMISClientTests(DMSMixin, TestCase):
    def test_run_sync_dryrun(self):
        self.cmis_client.sync(dryrun=True)

    def test_run_sync(self):
        result = self.cmis_client.sync()
        self.assertEqual(result.get('created'), 0)
        self.assertEqual(result.get('updated'), 0)
        self.assertEqual(result.get('deleted'), 0)
        self.assertEqual(result.get('security'), 0)
        self.assertGreater(result.get('failed'), 0)

    def test_run_sync_twice_at_the_same_time(self):
        ChangeLog.objects.create(token=1)

        with self.assertRaises(SyncException) as exc:
            self.cmis_client.sync()

        self.assertEqual(exc.exception.args, ('A synchronization process is already running.',))

    def test_run_sync_twice(self):
        result1 = self.cmis_client.sync()
        self.assertEqual(result1.get('created'), 0)
        self.assertEqual(result1.get('updated'), 0)
        self.assertEqual(result1.get('deleted'), 0)
        self.assertEqual(result1.get('security'), 0)
        self.assertGreater(result1.get('failed'), 0)

        result2 = self.cmis_client.sync()
        self.assertEqual(result2, {})



from django.test import TestCase

from drc_cmis.utils import (
    FolderConfig, get_cmis_object_id, get_cmis_object_id_parts
)

from .mixins import DMSMixin


class UtilsTests(DMSMixin, TestCase):
    def test_get_cmis_object_id_parts(self):
        parts = get_cmis_object_id_parts('workspace://SpacesStore/76038f79-ce1e-4116-a58d-eaa619117923')
        self.assertEqual(parts, ('76038f79-ce1e-4116-a58d-eaa619117923', None, None))

    def test_get_cmis_object_id_parts_with_version(self):
        parts = get_cmis_object_id_parts('workspace://SpacesStore/76038f79-ce1e-4116-a58d-eaa619117923;v1')
        self.assertEqual(parts, ('76038f79-ce1e-4116-a58d-eaa619117923', 'v1', None))

    def test_get_cmis_object_id_parts_with_path(self):
        parts = get_cmis_object_id_parts('SpacesStore/76038f79-ce1e-4116-a58d-eaa619117923')
        self.assertEqual(parts, ('76038f79-ce1e-4116-a58d-eaa619117923', None, 'SpacesStore'))

    def test_get_cmis_object_id_parts_with_version_and_path(self):
        parts = get_cmis_object_id_parts('SpacesStore/76038f79-ce1e-4116-a58d-eaa619117923;v1')
        self.assertEqual(parts, ('76038f79-ce1e-4116-a58d-eaa619117923', 'v1', 'SpacesStore'))

    def test_get_cmis_object_id(self):
        object_id = get_cmis_object_id('workspace://SpacesStore/76038f79-ce1e-4116-a58d-eaa619117923')
        self.assertEqual(object_id, '76038f79-ce1e-4116-a58d-eaa619117923')

    def test_folder_config_no_params(self):
        with self.assertRaises(AssertionError) as exc:
            folder_config = FolderConfig()

        self.assertEqual(exc.exception.args, ('Either type or name is required', ))

    def test_folder_config_with_type(self):
        folder_config = FolderConfig(type_='test')
        self.assertEqual(folder_config.__repr__(), "<FolderConfig type_='test' name=None>")

    def test_folder_config_with_name(self):
        folder_config = FolderConfig(name='test')
        self.assertEqual(folder_config.__repr__(), "<FolderConfig type_=None name='test'>")

    def test_folder_config_with_type_and_name(self):
        folder_config = FolderConfig(type_='test', name='test')
        self.assertEqual(folder_config.__repr__(), "<FolderConfig type_='test' name='test'>")



from datetime import datetime
from unittest import skipIf

from django.test import TestCase

import pytz
from cmislib.exceptions import UpdateConflictException

from drc_cmis.exceptions import DocumentConflictException
from drc_cmis.storage import BinaireInhoud

from ..factories import (
    DRCCMISConnectionFactory, EnkelvoudigInformatieObjectFactory
)
from ..mixins import DMSMixin
from .test_zaakdocument import get_correct_date


class CMISClientTests(DMSMixin, TestCase):
    def test_check_lock_status_unlocked(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)

        result = self.cmis_client.is_locked(document)
        self.assertFalse(result)

    def test_check_lock_status_locked(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)
        self.cmis_client.checkout(document)

        result = self.cmis_client.is_locked(document)
        self.assertTrue(result)

    def test_create_lock_update_flow(self):
        """
        Assert that it's possible to create an empty document, lock it for
        update and then effectively set the content thereby unlocking it.
        """
        self.cmis_client.creeer_zaakfolder(self.zaak_url)

        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, stream=document.inhoud)
        inhoud = BinaireInhoud(b'leaky abstraction...', filename='bestand.txt')

        # flow
        checkout_id, _checkout_by = self.cmis_client.checkout(document)  # lock for update
        # TODO: Broken here. Test not possible
        # with self.assertRaises(DocumentConflictException):
        self.cmis_client.update_zaakdocument(koppeling, checkout_id, inhoud=inhoud)

        filename, file_obj = self.cmis_client.geef_inhoud(document)

        # make assertions about the results
        self.assertEqual(filename, 'bestand.txt')
        self.assertEqual(file_obj.read(), b'leaky abstraction...')

        # verify expected props
        cmis_doc = self.cmis_client._get_cmis_doc(document)
        self.assertExpectedProps(cmis_doc, {
            'cmis:contentStreamFileName': 'bestand.txt',
            'cmis:contentStreamLength': 20,
            'cmis:contentStreamMimeType': 'application/binary',  # the default if it couldn't be determined
            # 'zsdms:dct.categorie': document.informatieobjecttype.informatieobjectcategorie,
            'zsdms:dct.omschrijving': document.informatieobjecttype,
            'zsdms:documentIdentificatie': str(document.identificatie),
            'zsdms:documentauteur': document.auteur,
            'zsdms:documentbeschrijving': document.beschrijving,
            'zsdms:documentcreatiedatum': get_correct_date(document.creatiedatum),
            # 'zsdms:documentformaat': None,
            'zsdms:documentLink': None,
            'zsdms:documentontvangstdatum': None,
            'zsdms:documentstatus': None,
            'zsdms:documenttaal': document.taal,
            'zsdms:documentversie': None,
            'zsdms:documentverzenddatum': None,
            'zsdms:vertrouwelijkaanduiding': document.vertrouwelijkheidaanduiding
        })

        # the doc must be unlocked after update, easy check -> lock it
        try:
            cmis_doc.checkout()
        except UpdateConflictException:
            self.fail("Could not lock document after update, it was already/still locked")



class DRCClient:
    """
    Abstract base class for DRC interaction.
    """

    TEMP_FOLDER_NAME = "_temp"
    TRASH_FOLDER = "Unfiled"

    def creeer_zaakfolder(self, zaak_url):
        raise NotImplementedError

    def maak_zaakdocument(self, koppeling, zaak_url=None, filename=None, sender=None):
        raise NotImplementedError

    def maak_zaakdocument_met_inhoud(
        self, koppeling, zaak_url=None, filename=None, sender=None, stream=None, content_type=None
    ):
        raise NotImplementedError

    def geef_inhoud(self, document):
        raise NotImplementedError

    def zet_inhoud(self, document, stream, content_type=None, checkout_id=None):
        raise NotImplementedError

    def relateer_aan_zaak(self, document, zaak_url):
        raise NotImplementedError

    def update_zaakdocument(self, document, checkout_id=None, inhoud=None):
        raise NotImplementedError

    def checkout(self, document):
        raise NotImplementedError

    def cancel_checkout(self, document, checkout_id):
        raise NotImplementedError

    def ontkoppel_zaakdocument(self, document, zaak_url):
        raise NotImplementedError

    def is_locked(self, document):
        raise NotImplementedError

    def verwijder_document(self, document):
        raise NotImplementedError

    def sync(self, dryrun=False):
        raise NotImplementedError



class DMSException(Exception):
    pass


class DocumentExistsError(DMSException):
    pass


class DocumentDoesNotExistError(DMSException):
    pass


class SyncException(DMSException):
    pass


class DocumentConflictException(DMSException):
    pass


class DocumentLockedException(DMSException):
    pass



from django.db import models

from drc_cmis import settings

from .choices import ChangeLogStatus
from .utils import get_model_value


class ChangeLog(models.Model):
    token = models.BigIntegerField()
    created_on = models.DateTimeField(auto_now_add=True, unique=True)
    status = models.CharField(max_length=20, choices=ChangeLogStatus.choices, default=ChangeLogStatus.in_progress)

    class Meta:
        verbose_name = "Changelog"
        verbose_name_plural = "Changelogs"
        ordering = ("created_on",)


class DRCCMISConnection(models.Model):
    cmis_object_id = models.TextField(help_text="CMIS storage object id, internal use only", blank=True)
    enkelvoudiginformatieobject = models.OneToOneField(
        settings.DRC_CMIS_ENKELVOUDIGINFORMATIEOBJECT, on_delete=models.CASCADE,
        related_name='cmisstorage'
    )

    CMIS_MAPPING = {
        "zsdms:documenttaal": "taal",
        "zsdms:documentLink": "link",
        "cmis:name": "titel",
        "zsdms:documentIdentificatie": "identificatie",
        "zsdms:documentcreatiedatum": "creatiedatum",
        "zsdms:documentontvangstdatum": "ontvangstdatum",
        "zsdms:documentbeschrijving": "beschrijving",
        "zsdms:documentverzenddatum": "verzenddatum",
        "zsdms:vertrouwelijkaanduiding": "vertrouwelijkheidaanduiding",
        "zsdms:documentauteur": "auteur",
        "zsdms:documentstatus": "status",
        "zsdms:dct.omschrijving": "informatieobjecttype",
    }

    def __str__(self):
        return f"CMIS koppeling voor {self.enkelvoudiginformatieobject}"

    def set_cmis_doc(self, cmis_doc):
        self.cmis_object_id = cmis_doc.getObjectId().rsplit(";")[0]
        self.save()

    def get_cmis_properties(self, allow_none=True):
        """
        Returns the CMIS properties as dict.

        :param allow_none: Converts `None` to  empty string if `False` (default).
        :return: The `dict` of CMIS properties.
        """
        result = {}
        for cmis_property, field_name in self.CMIS_MAPPING.items():
            # try:
            #     field_class = get_model_field(EnkelvoudigInformatieObject, field_name)
            # except FieldDoesNotExist:
            #     field_class = None

            val = get_model_value(self.enkelvoudiginformatieobject, field_name)
            if val is None:
                if not allow_none:
                    val = ""
            result[cmis_property] = val

        return result

    cmis_properties = property(get_cmis_properties)

    def update_cmis_properties(self, new_cmis_properties, commit=False):
        if not self.pk:
            raise ValueError("Cannot update CMIS properties on unsaved instance.")

        updated_objects = set()

        for cmis_property, _field_name in self.CMIS_MAPPING.items():
            if cmis_property not in new_cmis_properties:
                continue
            updated_objects.add(self)

        if commit:
            for obj in updated_objects:
                obj.save()

        return updated_objects



from uuid import uuid4

from django.db import models


class EnkelvoudigInformatieObject(models.Model):
    uuid = models.UUIDField(unique=True, default=uuid4)
    identificatie = models.CharField(max_length=40, default=uuid4)
    bronorganisatie = models.CharField(max_length=9)
    creatiedatum = models.DateField()
    titel = models.CharField(max_length=200)
    vertrouwelijkheidaanduiding = models.CharField(max_length=200)
    auteur = models.CharField(max_length=200)
    status = models.CharField(max_length=20)
    beschrijving = models.TextField(max_length=1000)
    ontvangstdatum = models.DateField(null=True, blank=True)
    verzenddatum = models.DateField(null=True, blank=True)
    indicatie_gebruiksrecht = models.NullBooleanField(blank=True, default=None)
    ondertekening_soort = models.CharField(max_length=10, blank=True)
    ondertekening_datum = models.DateField(blank=True, null=True)
    informatieobjecttype = models.URLField()
    formaat = models.CharField(max_length=255, blank=True)
    taal = models.CharField(max_length=255, blank=True)
    bestandsnaam = models.CharField(max_length=255, blank=True)
    inhoud = models.FileField(upload_to='uploads/%Y/%m/')
    link = models.URLField(max_length=200, blank=True)
    integriteit_algoritme = models.CharField(max_length=20, blank=True)
    integriteit_waarde = models.CharField(max_length=128, blank=True)
    integriteit_datum = models.DateField(null=True, blank=True)

    def __str__(self) -> str:
        return str(self.identificatie)



from io import BytesIO


class BinaireInhoud:
    def __init__(self, data, filename):
        self.data = data
        self.bestandsnaam = filename

    def to_cmis(self) -> BytesIO:
        if self.data is None:
            return None

        return BytesIO(self.data)



import os

PROJECT_DIR = os.path.dirname(__file__)
BASE_DIR = PROJECT_DIR  # setting present in new startproject

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

SITE_ID = 1

INSTALLED_APPS = [
    'drc_cmis',
    'tests.app',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEST_RUNNER = 'xmlrunner.extra.djangotestrunner.XMLTestRunner'
TEST_OUTPUT_DIR = 'reports'
TEST_OUTPUT_FILE_NAME = 'junit.xml'

ROOT_URLCONF = 'tests.urls'

DEBUG = True
SECRET_KEY = 'this-is-really-not-a-secret'
ALLOWED_HOSTS = ['localhost', '127.0.0.1']

STATIC_URL = '/static/'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_DIR, 'database.db'),
    }
}


STATIC_URL = '/static/'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'DIRS': [
            os.path.join(PROJECT_DIR, 'templates'),
        ],
        'OPTIONS': {
            'debug': DEBUG,
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.media',
            ],
        },
    },
]

#
# CMIS SETTINGS
#
# DRC_CMIS_CLIENT_URL = 'http://localhost:8080/alfresco/cmisatom'
DRC_CMIS_CLIENT_URL = 'http://localhost:8082/alfresco/cmisatom'
DRC_CMIS_CLIENT_USER = 'admin'
DRC_CMIS_CLIENT_USER_PASSWORD = 'admin'
DRC_CMIS_ENKELVOUDIGINFORMATIEOBJECT = 'app.EnkelvoudigInformatieObject'



from django.utils.translation import ugettext_lazy as _

from djchoices import ChoiceItem, DjangoChoices


class CMISChangeType(DjangoChoices):
    created = ChoiceItem("created", _("Created"))
    updated = ChoiceItem("updated", _("Updated"))
    deleted = ChoiceItem("deleted", _("Deleted"))
    security = ChoiceItem("security", _("Security"))


class CMISObjectType(DjangoChoices):
    zaken = ChoiceItem("F:zsdms:zaken", _("Zaken hoofd folder"))
    zaak_folder = ChoiceItem("F:zsdms:zaak", _("Zaak folder"))
    edc = ChoiceItem("D:zsdms:document", _("Enkelvoudig document"))


class CMISCapabilities(DjangoChoices):
    """
    http://docs.oasis-open.org/cmis/CMIS/v1.0/cmis-spec-v1.0.html
    """

    changes = ChoiceItem(
        "Changes", _('Indicates what level of changes (if any) the repository exposes via the "change log" service.')
    )
    all_versions_searchable = ChoiceItem(
        "AllVersionsSearchable",
        _(
            "Ability of the Repository to include all versions of document. If False, typically either the latest or the latest major version will be searchable."
        ),
    )
    content_stream_updatability = ChoiceItem(
        "ContentStreamUpdatability",
        _("Indicates the support a repository has for updating a document's content stream."),
    )
    pwc_updatable = ChoiceItem(
        "PWCUpdatable", _('Ability for an application to update the "Private Working Copy" of a checked-out document.')
    )
    pwc_searchable = ChoiceItem(
        "PWCSearchable",
        _(
            'Ability of the Repository to include the "Private Working Copy" of checked-out documents in query search scope; otherwise PWC\'s are not searchable'
        ),
    )
    unfiling = ChoiceItem(
        "Unfiling",
        _("Ability for an application to leave a document or other file-able object not filed in any folder."),
    )
    multifiling = ChoiceItem(
        "Multifiling",
        _("Ability for an application to file a document or other file-able object in more than one folder."),
    )
    version_specific_filing = ChoiceItem(
        "VersionSpecificFiling",
        _("Ability for an application to file individual versions (i.e., not all versions) of a document in a folder."),
    )
    renditions = ChoiceItem(
        "Renditions", _("Indicates whether or not the repository exposes renditions of document or folder objects.")
    )
    query = ChoiceItem("Query", _("Indicates the types of queries that the Repository has the ability to fulfill."))
    get_folder_tree = ChoiceItem(
        "GetFolderTree", _("Ability for an application to retrieve the folder tree via the getFolderTree service.")
    )
    acl = ChoiceItem("ACL", _("Indicates the level of support for ACLs by the repository."))
    join = ChoiceItem("Join", _(" Indicates the types of JOIN keywords that the Repository can fulfill in queries."))


class CMISCapabilityContentStreamUpdatability(DjangoChoices):
    none = ChoiceItem("none", _(" The content stream may never be updated."))
    anytime = ChoiceItem("anytime", _(" The content stream may be updated any time."))
    pwconly = ChoiceItem("pwconly", _(" The content stream may be updated only when checked out."))


class CMISCapabilityRenditions(DjangoChoices):
    read = ChoiceItem("read", _(" Renditions are provided by the repository and readable by the client."))
    none = ChoiceItem("none", _(" The repository does not expose renditions at all."))


class CMISCapabilityQuery(DjangoChoices):
    none = ChoiceItem("none", _(" No queries of any kind can be fulfilled."))
    metadataonly = ChoiceItem(
        "metadataonly",
        _(
            " Only queries that filter based on object properties can be fulfilled. Specifically, the CONTAINS() predicate function is not supported."
        ),
    )
    fulltextonly = ChoiceItem(
        "fulltextonly",
        _(
            " Only queries that filter based on the full-text content of documents can be fulfilled. Specifically, only the CONTAINS() predicate function can be included in the WHERE clause."
        ),
    )
    bothseparate = ChoiceItem(
        "bothseparate",
        _(
            " The repository can fulfill queries that filter EITHER on the full-text content of documents OR on their properties, but NOT if both types of filters are included in the same query."
        ),
    )
    bothcombined = ChoiceItem(
        "bothcombined",
        _(
            " The repository can fulfill queries that filter on both the full-text content of documents and their properties in the same query."
        ),
    )


class CMISCapabilityChanges(DjangoChoices):
    none = ChoiceItem("none", _(" The repository does not support the change log feature."))
    objectidsonly = ChoiceItem(
        "objectidsonly",
        _(
            " The change log can return only the ObjectIDs for changed objects in the repository and an indication of the type of change, not details of the actual change."
        ),
    )
    properties = ChoiceItem(
        "properties", _(" The change log can return properties and the ObjectID for the changed objects")
    )
    all = ChoiceItem(
        "all",
        _(
            " The change log can return the ObjectIDs for changed objects in the repository and more information about the actual change"
        ),
    )


class CMISCapabilityACL(DjangoChoices):
    none = ChoiceItem("none", _(" The repository does not support ACL services"))
    discover = ChoiceItem("discover", _(" The repository supports discovery of ACLs (getACL and other services)"))
    manage = ChoiceItem(
        "manage", _(" The repository supports discovery of ACLs AND applying ACLs (getACL and applyACL services)")
    )


class ChangeLogStatus(DjangoChoices):
    completed = ChoiceItem("completed", _("Completed"))
    in_progress = ChoiceItem("in_progress", _("In progress"))



from django.urls import reverse

from drc.backend.abstract import BaseDRCStorageBackend

from drc_cmis import settings

from .client import default_client


class CMISDRCStorageBackend(BaseDRCStorageBackend):
    """
    This is the backend that is used to store the documents in a CMIS compatible backend.
    """

    def get_folder(self, zaak_url):
        # Folders will not be supported for now
        pass

    def create_folder(self, zaak_url):
        # Folders will not be supported for now
        pass

    def rename_folder(self, old_zaak_url, new_zaak_url):
        # Folders will not be supported for now
        pass

    def remove_folder(self, zaak_url):
        # Folders will not be supported for now
        pass

    def get_document(self, enkelvoudiginformatieobject):
        try:
            enkelvoudiginformatieobject.koppeling
        except AttributeError:
            return None
        else:
            return reverse('cmis:cmis-document-download', kwargs={'inhoud': enkelvoudiginformatieobject.identificatie})

    def create_document(self, enkelvoudiginformatieobject, bestand=None, link=None):
        from .models import DRCCMISConnection
        connection = DRCCMISConnection.objects.create(
            enkelvoudiginformatieobject=enkelvoudiginformatieobject, cmis_object_id=""
        )

        cmis_doc = default_client.maak_zaakdocument_met_inhoud(
            koppeling=connection,
            zaak_url=None,
            filename=None,
            sender=settings.DRC_CMIS_SENDER_PROPERTY,
            stream=bestand,
        )
        connection.cmis_object_id = cmis_doc.getObjectId().rsplit(";")[0]
        connection.save()

    def update_document(self, enkelvoudiginformatieobject, updated_values, bestand=None, link=None):
        if not hasattr(enkelvoudiginformatieobject, 'cmisstorage'):
            raise AttributeError('This document is not connected to CMIS')
        default_client.update_zaakdocument(enkelvoudiginformatieobject.cmisstorage)

    def remove_document(self, enkelvoudiginformatieobject):
        raise NotImplementedError()

    def move_document(self, enkelvoudiginformatieobject):
        raise NotImplementedError()



from django.contrib import admin
from django.urls import include, path

urlpatterns = [
    path('admin/', admin.site.urls),
    path('cmis/', include(('drc_cmis.urls', 'drc_cmis'), namespace='cmis')),
]



from django.test import override_settings

from cmislib.exceptions import ObjectNotFoundException

from drc_cmis import settings
from drc_cmis.client import default_client


class DMSMixin:
    @override_settings(DRC_CMIS_TEMP_FOLDER_NAME='/_temp')
    def setUp(self):
        super().setUp()

        self.cmis_client = default_client
        self.addCleanup(lambda: self._removeTree('/Zaken'))
        self.addCleanup(lambda: self._removeTree('/Sites/archief/documentLibrary'))
        self.addCleanup(lambda: self._removeTree('/_temp'))
        self.addCleanup(lambda: self._removeTree('/Unfiled'))
        self.addCleanup(lambda: self._removeTree('/enkelvoudiginformatieobjecten'))

        self.zaak_url = 'http://zaak.nl/locatie'

    def _removeTree(self, path):
        try:
            root_folder = self.cmis_client._repo.getObjectByPath(path)
        except ObjectNotFoundException:
            return
        root_folder.deleteTree()

    def assertExpectedProps(self, obj, expected: dict):
        for prop, expected_value in expected.items():
            with self.subTest(prop=prop, expected_value=expected_value):
                self.assertEqual(obj.properties[prop], expected_value, msg="prop: {}".format(prop))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import find_packages, setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'django_choices',
    'django',
    'cmislib-maykin',
]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="maykinmedia",
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
    ],
    description="A CMIS backend for gemma-documentregistratiecomponent",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='drc_cmis',
    name='drc_cmis',
    packages=find_packages(include=['drc_cmis']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/gemeenteutrecht/drc_cmis',
    version='0.0.2',
    zip_safe=False,
)



from datetime import date, datetime
from io import BytesIO
from unittest import skipIf

from django.test import TestCase, override_settings

import pytz

from drc_cmis.exceptions import DocumentExistsError

from ..factories import (
    DRCCMISConnectionFactory, EnkelvoudigInformatieObjectFactory
)
from ..mixins import DMSMixin


def get_correct_date(document_date):
    if isinstance(document_date, str):
        return datetime.combine(datetime.strptime(document_date, '%Y-%m-%d').date(), datetime.min.time()).replace(tzinfo=pytz.utc)
    else:
        return datetime.combine(document_date, datetime.min.time()).replace(tzinfo=pytz.utc)


class CMISClientTests(DMSMixin, TestCase):
    def test_maak_zaakdocument(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        koppeling = DRCCMISConnectionFactory.create()

        # cmis_doc = self.cmis_client._get_cmis_doc(document)
        cmis_doc = self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)
        koppeling.set_cmis_doc(cmis_doc)
        # verify that it identifications are unique
        with self.assertRaises(DocumentExistsError):
            self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)

        koppeling.refresh_from_db()
        document = koppeling.enkelvoudiginformatieobject
        # verify expected props
        self.assertExpectedProps(cmis_doc, {
            'cmis:contentStreamFileName': document.titel,
            'cmis:contentStreamLength': 0,
            'cmis:contentStreamMimeType': 'application/binary',
            'zsdms:dct.omschrijving': document.informatieobjecttype,
            'zsdms:documentIdentificatie': str(document.identificatie),
            'zsdms:documentauteur': document.auteur,
            'zsdms:documentbeschrijving': document.beschrijving,
            'zsdms:documentcreatiedatum': get_correct_date(document.creatiedatum),
            'zsdms:documentLink': None,
            'zsdms:documentontvangstdatum': document.ontvangstdatum,
            'zsdms:documentstatus': None,
            'zsdms:documenttaal': document.taal,
            'zsdms:documentversie': None,
            'zsdms:documentverzenddatum': None,
            'zsdms:vertrouwelijkaanduiding': document.vertrouwelijkheidaanduiding
        })

        koppeling.refresh_from_db()
        self.assertEqual(
            koppeling.cmis_object_id,
            cmis_doc.properties['cmis:objectId'].rsplit(';')[0]
        )

    def test_maak_zaakdocument_met_gevulde_inhoud(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)

        koppeling = DRCCMISConnectionFactory.create()

        cmis_doc = self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, self.zaak_url, stream=BytesIO(b'test'))
        koppeling.set_cmis_doc(cmis_doc)
        document = koppeling.enkelvoudiginformatieobject
        self.assertExpectedProps(cmis_doc, {
            'cmis:contentStreamFileName': document.titel,
            'cmis:contentStreamLength': 4,
            'cmis:contentStreamMimeType': 'application/binary',
            'zsdms:dct.omschrijving': document.informatieobjecttype,
            'zsdms:documentIdentificatie': str(document.identificatie),
            'zsdms:documentauteur': document.auteur,
            'zsdms:documentbeschrijving': document.beschrijving,
            'zsdms:documentcreatiedatum': get_correct_date(document.creatiedatum),
            'zsdms:documentLink': None,
            'zsdms:documentontvangstdatum': document.ontvangstdatum,
            'zsdms:documentstatus': None,
            'zsdms:documenttaal': document.taal,
            'zsdms:documentversie': None,
            'zsdms:documentverzenddatum': None,
            'zsdms:vertrouwelijkaanduiding': document.vertrouwelijkheidaanduiding
        })

        document.refresh_from_db()
        self.assertEqual(
            koppeling.cmis_object_id,
            cmis_doc.properties['cmis:objectId'].rsplit(';')[0]
        )

    @override_settings(DRC_CMIS_SENDER_PROPERTY='zsdms:documentauteur')
    def test_maak_zaakdocument_met_sender_property(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)

        koppeling = DRCCMISConnectionFactory.create()

        cmis_doc = self.cmis_client.maak_zaakdocument_met_inhoud(koppeling, self.zaak_url, sender='maykin', stream=BytesIO(b'test'))
        koppeling.set_cmis_doc(cmis_doc)

        document = koppeling.enkelvoudiginformatieobject
        self.assertExpectedProps(cmis_doc, {
            'cmis:contentStreamFileName': document.titel,
            'cmis:contentStreamLength': 4,
            'cmis:contentStreamMimeType': 'application/binary',
            'zsdms:dct.omschrijving': document.informatieobjecttype,
            'zsdms:documentIdentificatie': str(document.identificatie),
            'zsdms:documentauteur': 'maykin',  # overridden by the sender
            'zsdms:documentbeschrijving': document.beschrijving,
            'zsdms:documentcreatiedatum': get_correct_date(document.creatiedatum),
            'zsdms:documentLink': None,
            'zsdms:documentontvangstdatum': document.ontvangstdatum,
            'zsdms:documentstatus': None,
            'zsdms:documenttaal': document.taal,
            'zsdms:documentversie': None,
            'zsdms:documentverzenddatum': None,
            'zsdms:vertrouwelijkaanduiding': document.vertrouwelijkheidaanduiding
        })

        koppeling.refresh_from_db()
        self.assertEqual(
            koppeling.cmis_object_id,
            cmis_doc.properties['cmis:objectId'].rsplit(';')[0]
        )

    def test_lees_document(self):
        """
        Ref #83: geefZaakdocumentLezen vraagt een kopie van het bestand op.

        Van het bestand uit het DMS wordt opgevraagd: inhoud, bestandsnaam.
        """
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        koppeling = DRCCMISConnectionFactory.create()
        document = koppeling.enkelvoudiginformatieobject
        cmis_doc = self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)
        koppeling.set_cmis_doc(cmis_doc)

        # empty by default
        filename, file_obj = self.cmis_client.geef_inhoud(document)

        self.assertEqual(filename, document.titel)
        self.assertEqual(file_obj.read(), b'')

        cmis_doc.setContentStream(BytesIO(b'some content'), 'text/plain')

        filename, file_obj = self.cmis_client.geef_inhoud(document)

        self.assertEqual(filename, document.titel)
        self.assertEqual(file_obj.read(), b'some content')

    def test_lees_document_bestaad_niet(self):
        """
        Ref #83: geefZaakdocumentLezen vraagt een kopie van het bestand op.

        Van het bestand uit het DMS wordt opgevraagd: inhoud, bestandsnaam.
        """
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        document = EnkelvoudigInformatieObjectFactory.build(identificatie='123456')

        # empty by default
        filename, file_obj = self.cmis_client.geef_inhoud(document)

        self.assertEqual(filename, None)
        self.assertEqual(file_obj.read(), b'')

    def test_voeg_zaakdocument_toe(self):
        """
        4.3.4.3 Interactie tussen ZS en DMS

        Het ZS zorgt ervoor dat het document dat is aangeboden door de DSC wordt vastgelegd in het DMS.
        Hiervoor maakt het ZS gebruik van de CMIS-services die aangeboden worden door het DMS. Hierbij
        gelden de volgende eisen:
        - Het document wordt gerelateerd aan de juiste Zaakfolder (Zie 5.1)
        - Het document wordt opgeslagen als objecttype EDC (Zie 5.2)
        - Minimaal de vereiste metadata voor een EDC wordt vastgelegd in de daarvoor gedefinieerde
        objectproperties. In Tabel 3 is een mapping aangegeven tussen de StUF-ZKN-elementen en
        CMIS-objectproperties.
        """
        koppeling = DRCCMISConnectionFactory.create()
        self.cmis_client.maak_zaakdocument(koppeling)
        koppeling.refresh_from_db()

        result = self.cmis_client.zet_inhoud(koppeling.enkelvoudiginformatieobject, BytesIO(b'some content'), content_type='text/plain')

        self.assertIsNone(result)
        filename, file_obj = self.cmis_client.geef_inhoud(koppeling.enkelvoudiginformatieobject)
        self.assertEqual(file_obj.read(), b'some content')
        self.assertEqual(filename, koppeling.enkelvoudiginformatieobject.titel)

    def test_relateer_aan_zaak(self):
        koppeling = DRCCMISConnectionFactory.create()
        zaak_folder = self.cmis_client.creeer_zaakfolder(self.zaak_url)
        self.cmis_client.maak_zaakdocument(koppeling)
        koppeling.refresh_from_db()

        result = self.cmis_client.relateer_aan_zaak(koppeling.enkelvoudiginformatieobject, self.zaak_url)
        self.assertIsNone(result)

        cmis_doc = self.cmis_client._get_cmis_doc(koppeling.enkelvoudiginformatieobject)
        parents = [parent.id for parent in cmis_doc.getObjectParents()]
        self.assertEqual(parents, [zaak_folder.id])

    def test_ontkoppel_zaakdocument(self):
        cmis_folder = self.cmis_client.creeer_zaakfolder(self.zaak_url)
        koppeling = DRCCMISConnectionFactory.create()
        self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)
        result = self.cmis_client.ontkoppel_zaakdocument(koppeling.enkelvoudiginformatieobject, self.zaak_url)
        self.assertIsNone(result)

        # check that the zaakfolder is empty
        self.assertFalse(cmis_folder.getChildren())

    def test_verwijder_document(self):
        zaak_folder = self.cmis_client.creeer_zaakfolder(self.zaak_url)
        koppeling = DRCCMISConnectionFactory.create()
        self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)

        result = self.cmis_client.verwijder_document(koppeling.enkelvoudiginformatieobject)

        self.assertIsNone(result)
        # check that it's gone
        trash_folder, _ = self.cmis_client._get_or_create_folder(self.cmis_client.TRASH_FOLDER)
        self.assertEqual(len(trash_folder.getChildren()), 0)
        self.assertEqual(len(zaak_folder.getChildren()), 0)



from django.test import TestCase

from cmislib.exceptions import CmisException

from drc_cmis.client import CMISDRCClient


class TestClient(CMISDRCClient):
    def __init__(self):
        super().__init__(
            url='http://test.com/',
            user='test',
            password='test',
        )


class TestClientTests(TestCase):
    def test_init_test_client(self):
        with self.assertRaises(CmisException):
            TestClient()



import logging
from collections import OrderedDict
from io import BytesIO

from django.apps import apps
from django.core.exceptions import ImproperlyConfigured
from django.db import transaction
from django.utils.functional import LazyObject
from django.utils.module_loading import import_string
from django.utils.text import slugify

from cmislib import CmisClient
from cmislib.exceptions import ObjectNotFoundException, UpdateConflictException

from drc_cmis import settings

from .abstract import DRCClient
from .choices import ChangeLogStatus, CMISChangeType, CMISObjectType
from .exceptions import (
    DocumentConflictException, DocumentDoesNotExistError, DocumentExistsError,
    DocumentLockedException, SyncException
)
from .query import CMISQuery
from .utils import get_cmis_object_id

logger = logging.getLogger(__name__)


class CMISDRCClient(DRCClient):
    """
    DRC client implementation using the CMIS protocol.
    """

    document_query = CMISQuery("SELECT * FROM zsdms:document WHERE zsdms:documentIdentificatie = '%s'")
    TEMP_FOLDER_NAME = settings.DRC_CMIS_TEMP_FOLDER_NAME

    def __init__(self, url=None, user=None, password=None):
        """
        Connect to the CMIS repository and store the root folder for further
        operations.

        :param url: string, CMIS provider url.
        :param user: string, username to login on the document store
        :param password: string, password to login on the document store
        """
        if url is None:
            url = settings.DRC_CMIS_CLIENT_URL
        if user is None:
            user = settings.DRC_CMIS_CLIENT_USER
        if password is None:
            password = settings.DRC_CMIS_CLIENT_USER_PASSWORD
        _client = CmisClient(url, user, password)
        self._repo = _client.getDefaultRepository()
        self._root_folder = self._repo.getObjectByPath("/")
        self.upload_to = import_string(settings.DRC_CMIS_UPLOAD_TO)

    def _get_or_create_folder(self, name, properties=None, parent=None):
        """
        Get or create the folder with :param:`name` in :param:`parent`.

        :param name: string, the name of the folder to create.
        :param properties: dictionary with cmis and/or custom properties to
          pass to the folder object
        :param parent: parent folder to create the folder in as subfolder.
          Defaults to the root folder
        :return: a tuple of (folder, boolean) where the folder is the retrieved or created folder, and
          the boolean indicates whether the folder was created or not.
        """
        if parent is None:
            parent = self._root_folder
        existing = next((child for child in parent.getChildren() if child.name == name), None)
        if existing is not None:
            return (existing, False)
        return (parent.createFolder(name, properties=properties or {}), True)

    def get_folder_name(self, zaak_url, folder_config):
        name = ""
        if folder_config.type == CMISObjectType.zaak_folder:
            name = slugify(zaak_url)
        else:
            if not folder_config.name:
                raise ValueError(("Could not determine a folder name for zaak {}").format(slugify(zaak_url)))
        return folder_config.name or name

    def _get_zaakfolder(self, zaak_url):
        upload_to = self.upload_to(zaak_url)
        bits = [self.get_folder_name(zaak_url, folder_config) for folder_config in upload_to]
        path = "/" + ("/").join(bits)
        return self._repo.getObjectByPath(path)

    def _get_cmis_doc(self, document, checkout_id=None):
        """
        Given a document instance, retrieve the underlying AtomPubDocument object.

        :param document: :class:`InformatieObject` instance.
        :return: :class:`AtomPubDocument` object
        """
        query = self.document_query(document.identificatie)
        result_set = self._repo.query(query)
        unpacked_result_set = [item for item in result_set]
        if not unpacked_result_set:
            error_string = "Document met identificatie {} bestaat niet in het DRC".format(document.identificatie)
            raise DocumentDoesNotExistError(error_string)
        doc = unpacked_result_set[0]
        doc = doc.getLatestVersion()
        if checkout_id is not None:
            pwc = doc.getPrivateWorkingCopy()
            if not pwc or not pwc.properties["cmis:versionSeriesCheckedOutId"] == checkout_id:
                raise DocumentConflictException("Foutieve 'pwc id' meegestuurd")
        return doc

    def _build_cmis_doc_properties(self, connection, filename=None):
        properties = connection.get_cmis_properties()
        properties["cmis:objectTypeId"] = CMISObjectType.edc
        if filename is not None:
            properties["cmis:name"] = filename
        return properties

    def creeer_zaakfolder(self, zaak_url):
        """
        Maak de zaak folder aan in het DRC.

        :param zaak_url: Een link naar de zaak.
        :return: :class:`cmslib.atompub_binding.AtomPubFolder` object - de
          cmslib representatie van de (aangemaakte) zaakmap.
        """
        upload_to = self.upload_to(zaak_url)
        for folder_config in upload_to:
            if folder_config.type == CMISObjectType.zaken:
                folder_config.type = "cmis:folder"

        parent = None
        for folder_config in upload_to:
            properties = {"cmis:objectTypeId": folder_config.type} if folder_config.type else {}
            name = self.get_folder_name(zaak_url, folder_config)
            parent, _ = self._get_or_create_folder(name, properties, parent=parent)

        zaak_folder = parent
        return zaak_folder

    def maak_zaakdocument(self, koppeling, zaak_url=None, filename=None, sender=None):
        return self.maak_zaakdocument_met_inhoud(koppeling, zaak_url, filename, sender)

    def maak_zaakdocument_met_inhoud(
        self, koppeling, zaak_url=None, filename=None, sender=None, stream=None, content_type=None
    ):
        """
        :param zaak_url: TODO
        :param document: EnkelvoudigInformatieObject instantie die de
          meta-informatie van het document bevat
        :param filename: Bestandsnaam van het aan te maken document.
        :param sender: De afzender.
        :param stream: Inhoud van het document.
        :param content_type: Aanduiding van het document type.

        :return: AtomPubDocument instance die aangemaakt werd.
        :raises: DocumentExistsError wanneer er al een document met dezelfde
            identificatie bestaat, binnen de zaakfolder.
        """
        try:
            self._get_cmis_doc(koppeling.enkelvoudiginformatieobject)
        except DocumentDoesNotExistError:
            pass
        else:
            error_string = "Document identificatie {} is niet uniek".format(
                koppeling.enkelvoudiginformatieobject.identificatie
            )
            raise DocumentExistsError(error_string)

        if stream is None:
            stream = BytesIO()
        if zaak_url is None:
            zaakfolder, _ = self._get_or_create_folder(self.TEMP_FOLDER_NAME)
        else:
            zaakfolder = self._get_zaakfolder(zaak_url)

        properties = self._build_cmis_doc_properties(koppeling, filename=filename)
        if settings.DRC_CMIS_SENDER_PROPERTY:
            properties[settings.DRC_CMIS_SENDER_PROPERTY] = sender

        _doc = self._repo.createDocument(
            name=koppeling.enkelvoudiginformatieobject.titel,
            properties=properties,
            contentFile=stream,
            contentType=content_type,
            parentFolder=zaakfolder,
        )
        return _doc

    def geef_inhoud(self, document):
        """
        Retrieve the document via its identifier from the DRC.

        :param document: EnkelvoudigInformatieObject instance
        :return: tuple of (filename, BytesIO()) with the stream filename and the binary content
        """
        try:
            doc = self._get_cmis_doc(document)
        except DocumentDoesNotExistError:
            return (None, BytesIO())

        filename = doc.properties["cmis:name"]
        empty = doc.properties["cmis:contentStreamId"] is None
        if empty:
            return (filename, BytesIO())
        return (filename, doc.getContentStream())

    def zet_inhoud(self, document, stream, content_type=None, checkout_id=None):
        """
        Calls setContentStream to fill the contents of an existing document. This will update the
        version of the document in the DRC.

        :param document: EnkelvoudigInformatieObject instance
        :param stream: Inhoud van het document.
        :param content_type: Aanduiding van het document type.
        :param checkout_id:
        """
        cmis_doc = self._get_cmis_doc(document, checkout_id=checkout_id)
        if checkout_id:
            cmis_doc = cmis_doc.getPrivateWorkingCopy()
        cmis_doc.setContentStream(stream, content_type)

    def update_zaakdocument(self, koppeling, checkout_id=None, inhoud=None):
        cmis_doc = self._get_cmis_doc(koppeling.enkelvoudiginformatieobject, checkout_id=checkout_id)

        if checkout_id:
            cmis_doc = cmis_doc.getPrivateWorkingCopy()
        # build up the properties
        current_properties = cmis_doc.properties
        new_properties = self._build_cmis_doc_properties(koppeling, filename=inhoud.bestandsnaam if inhoud else None)
        diff_properties = {
            key: value
            for key, value in new_properties.items()
            if current_properties.get(key) != new_properties.get(key)
        }

        try:
            cmis_doc.updateProperties(diff_properties)
        except UpdateConflictException as exc:
            # Node locked!
            raise DocumentConflictException from exc

        if inhoud is not None:
            content = inhoud.to_cmis()
            self.zet_inhoud(koppeling.enkelvoudiginformatieobject, content, None, checkout_id=checkout_id)

        # all went well so far, so if we have a checkout_id, we must check the document back in
        if checkout_id:
            cmis_doc.checkin()

    def relateer_aan_zaak(self, document, zaak_url):
        """
        Wijs het document aan :param:`zaak` toe.

        Verplaatst het document van de huidige folder naar de zaakfolder.
        """
        cmis_doc = self._get_cmis_doc(document)
        zaakfolder = self._get_zaakfolder(zaak_url)
        parent = [parent for parent in cmis_doc.getObjectParents()][0]
        cmis_doc.move(parent, zaakfolder)

    def checkout(self, document):
        """
        Checkout (lock) the requested document and return the PWC ID + check out username.

        :param document: :class:`EnkelvoudigInformatieObject` instance.
        """
        cmis_doc = self._get_cmis_doc(document)
        try:
            pwc = cmis_doc.checkout()
        except UpdateConflictException as exc:
            raise DocumentLockedException("Document was already checked out") from exc

        pwc.reload()
        checkout_id = pwc.properties["cmis:versionSeriesCheckedOutId"]
        checkout_by = pwc.properties["cmis:versionSeriesCheckedOutBy"]
        return (checkout_id, checkout_by)

    def cancel_checkout(self, document, checkout_id):
        cmis_doc = self._get_cmis_doc(document, checkout_id=checkout_id)
        cmis_doc.cancelCheckout()

    def ontkoppel_zaakdocument(self, document, zaak_url):
        cmis_doc = self._get_cmis_doc(document)
        cmis_folder = self._get_zaakfolder(zaak_url)
        trash_folder, _ = self._get_or_create_folder(self.TRASH_FOLDER)
        cmis_doc.move(cmis_folder, trash_folder)

    def is_locked(self, document):
        cmis_doc = self._get_cmis_doc(document)
        pwc = cmis_doc.getPrivateWorkingCopy()
        return pwc is not None

    def verwijder_document(self, document):
        cmis_doc = self._get_cmis_doc(document)
        cmis_doc.delete()

    @transaction.atomic
    def sync(self, dryrun=False):
        """
        De zaakdocument registratie in het DRC wordt gesynchroniseerd met het
        ZRC door gebruik te maken van de CMIS-changelog. Het ZRC vraagt deze op
        bij het DRC door gebruik te maken van de CMISservice
        getContentChanges(), die het DRC biedt. Het ZRC dient door middel van de
        latestChangeLogToken te bepalen welke wijzigingen in de CMIS-changelog
        nog niet zijn verwerkt in het ZRC. Indien er wijzigingen zijn die nog
        niet zijn verwerkt in het ZRC dienen deze alsnog door het ZRC verwerkt te
        worden.

        Zie: ZDS 1.2, paragraaf 4.4

        De sync-functie, realiseert ook "Koppel Zaakdocument aan Zaak":

        Een reeds bestaand document wordt relevant voor een lopende zaak.

        De "Koppel Zaakdocument aan Zaak"-service biedt de mogelijkheid aan
        DSC's om een "los" document achteraf aan een zaak te koppelen waardoor
        het een zaakgerelateerd document wordt. Het betreft hier documenten
        die reeds bestonden en in het DRC waren vastgelegd voordat een ZAAK is
        ontstaan.

        Een document wordt binnen het DRC gekoppeld aan een lopende zaak door
        het document te relateren aan een Zaakfolder-object.

        Zie: ZDS 1.2, paragraaf 5.4.2

        :param dryrun: Retrieves all content changes from the DRC but doesn't
                       update the ZRC.
        :return: A `OrderedDict` with all `CMISChangeType`s as key and the
                 number of actions as value.
        """
        from .models import ChangeLog, DRCCMISConnection
        EnkelvoudigInformatieObject = apps.get_model(*settings.DRC_CMIS_ENKELVOUDIGINFORMATIEOBJECT.split("."))
        self._repo.reload()
        try:
            dms_change_log_token = int(self._repo.info["latestChangeLogToken"])
        except KeyError:
            raise ImproperlyConfigured("Could not retrieve the latest change log token from the DRC.")

        if not dryrun:
            change_log = ChangeLog.objects.create(token=dms_change_log_token)
            if ((ChangeLog.objects.exclude(pk=change_log.pk)).filter(status=ChangeLogStatus.in_progress)).count() > 0:
                change_log.delete()
                raise SyncException("A synchronization process is already running.")
            else:
                pass
                # change_log = None
            last_change_log = (ChangeLog.objects.filter(status=ChangeLogStatus.completed)).last()
            last_zs_change_log_token = last_change_log.token if last_change_log else 0
            max_items = dms_change_log_token - last_zs_change_log_token
            if max_items < 0:
                raise SyncException("The DRC change log token is older than our records.")
            else:
                if max_items == 0:
                    return {}
                created, updated, deleted, security, failed = (0, 0, 0, 0, 0)
                cache = set()
                result_set = self._repo.getContentChanges(
                    changeLogToken=last_zs_change_log_token, includeProperties=True, maxItems=max_items
                )

                for change_entry in result_set:
                    change_type = change_entry.changeType
                    object_id = get_cmis_object_id(change_entry.objectId)
                    cache_key = ("{}-{}").format(object_id, change_type)
                    if cache_key in cache:
                        continue
                    cache.add(cache_key)
                    try:
                        if change_type in [CMISChangeType.created, CMISChangeType.updated]:
                            try:
                                dms_document = self._repo.getObject(object_id)
                            except ObjectNotFoundException:
                                logger.error(
                                    "[%s-%s] Object was %s but could not be found in the DRC.",
                                    change_entry.id,
                                    object_id,
                                    change_type,
                                )
                                failed += 1
                                continue

                            dms_object_type = dms_document.properties.get("cmis:objectTypeId")
                            if dms_object_type == CMISObjectType.edc:
                                if change_type == CMISChangeType.updated:
                                    try:
                                        zs_document_id = dms_document.properties.get("zsdms:documentIdentificatie")
                                        edc = EnkelvoudigInformatieObject.objects.get(identificatie=zs_document_id)
                                    except EnkelvoudigInformatieObject.DoesNotExist:
                                        logger.error(
                                            "[%s-%s] Object was %s but could not be found in the ZRC.",
                                            change_entry.id,
                                            object_id,
                                            change_type,
                                        )
                                        failed += 1
                                    else:
                                        edc.update_cmis_properties(dms_document.properties, commit=not dryrun)
                                        updated += 1

                                else:
                                    dms_document.getPaths()[0].split("/")[-2]
                                    created += 1
                        else:
                            if change_type == CMISChangeType.deleted:
                                if not dryrun:
                                    delete_count = (DRCCMISConnection.objects.filter(cmis_object_id=object_id)).delete()
                                    if delete_count[0] == 0:
                                        logger.warning(
                                            "[%s-%s] Object was %s but could not be found in the ZRC.",
                                            change_entry.id,
                                            object_id,
                                            change_type,
                                        )
                                        failed += 1
                                    else:
                                        deleted += 1
                                else:
                                    if change_type == CMISChangeType.security:
                                        logger.info(
                                            "[%s-%s] Security changes are not processed.", change_entry.id, object_id
                                        )
                                        security += 1
                                    else:
                                        logger.error(
                                            "[%s-%s] Unsupported change type: %s",
                                            change_entry.id,
                                            object_id,
                                            change_type,
                                        )
                                        failed += 1
                    except Exception as e:
                        failed += 1
                        logger.exception(
                            '[%s-%s] Could not process "%s" in ZRC: %s',
                            change_entry.id,
                            object_id,
                            change_type,
                            e,
                            exc_info=True,
                        )

                if not dryrun and change_log:
                    change_log.status = ChangeLogStatus.completed
                    change_log.save()
                return OrderedDict(
                    [
                        (CMISChangeType.created, created),
                        (CMISChangeType.updated, updated),
                        (CMISChangeType.deleted, deleted),
                        (CMISChangeType.security, security),
                        ("failed", failed),
                    ]
                )


class DefaultClient(LazyObject):
    def _setup(self):
        client_cls = import_string(settings.DRC_CMIS_CLIENT_CLASS)
        self._wrapped = client_cls()


default_client = DefaultClient()



import sys

from django.conf import settings


class _Settings(object):
    @property
    def DRC_CMIS_CLIENT_URL(self):
        return getattr(settings, "DRC_CMIS_CLIENT_URL")

    @property
    def DRC_CMIS_CLIENT_USER(self):
        return getattr(settings, "DRC_CMIS_CLIENT_USER")

    @property
    def DRC_CMIS_CLIENT_USER_PASSWORD(self):
        return getattr(settings, "DRC_CMIS_CLIENT_USER_PASSWORD")

    @property
    def DRC_CMIS_SENDER_PROPERTY(self):
        return getattr(settings, "DRC_CMIS_SENDER_PROPERTY", None)

    @property
    def DRC_CMIS_UPLOAD_TO(self):
        return getattr(settings, "DRC_CMIS_UPLOAD_TO", "drc_cmis.utils.upload_to")

    @property
    def DRC_CMIS_CLIENT_CLASS(self):
        return getattr(settings, "DRC_CMIS_CLIENT_CLASS", "drc_cmis.client.CMISDRCClient")

    @property
    def DRC_CMIS_ENKELVOUDIGINFORMATIEOBJECT(self):
        return getattr(settings, "DRC_CMIS_ENKELVOUDIGINFORMATIEOBJECT", "datamodel.EnkelvoudigInformatieObject")

    @property
    def DRC_CMIS_TEMP_FOLDER_NAME(self):
        return getattr(settings, "DRC_CMIS_TEMP_FOLDER_NAME", "enkelvoudiginformatieobjecten")

    def __getattr__(self, name):
        return globals()[name]


# other parts of itun that you WANT to code in
# module-ish ways
sys.modules[__name__] = _Settings()



from django.test import TestCase

from .factories import DRCCMISConnectionFactory


class ModelTests(TestCase):
    def test_get_cmis_properties(self):
        koppeling = DRCCMISConnectionFactory()
        document = koppeling.enkelvoudiginformatieobject
        self.assertEqual(koppeling.get_cmis_properties(), {
            'zsdms:documenttaal': document.taal,
            'zsdms:documentLink': '',
            'cmis:name': document.titel,
            'zsdms:documentIdentificatie': document.identificatie,
            'zsdms:documentcreatiedatum': document.creatiedatum,
            'zsdms:documentontvangstdatum': None,
            'zsdms:documentbeschrijving': document.beschrijving,
            'zsdms:documentverzenddatum': None,
            'zsdms:vertrouwelijkaanduiding': document.vertrouwelijkheidaanduiding,
            'zsdms:documentauteur': document.auteur,
            'zsdms:documentstatus': '',
            'zsdms:dct.omschrijving': 'https://example.com/ztc/api/v1/catalogus/1/informatieobjecttype/1'
        })

    def test_get_cmis_properties_with_mapping(self):
        koppeling = DRCCMISConnectionFactory()
        koppeling.CMIS_MAPPING = {'anders': 'taal'}
        self.assertEqual(koppeling.get_cmis_properties(), {'anders': koppeling.enkelvoudiginformatieobject.taal})

    def test_get_cmis_properties_with_none_values(self):
        koppeling = DRCCMISConnectionFactory()
        koppeling.CMIS_MAPPING = {'anders': 'ontvangstdatum'}
        self.assertEqual(koppeling.get_cmis_properties(), {'anders': None})

    def test_get_cmis_properties_with_none_values_allow_none_false(self):
        koppeling = DRCCMISConnectionFactory()
        koppeling.CMIS_MAPPING = {'anders': 'ontvangstdatum'}
        self.assertEqual(koppeling.get_cmis_properties(allow_none=False), {'anders': ''})

    def test_update_cmis_properties(self):
        koppeling = DRCCMISConnectionFactory()
        koppeling.update_cmis_properties({})



from unittest import skipIf

from django.test import TestCase

from cmislib.exceptions import UpdateConflictException

from drc_cmis.exceptions import (
    DocumentConflictException, DocumentLockedException
)

from ..factories import (
    DRCCMISConnectionFactory, EnkelvoudigInformatieObjectFactory
)
from ..mixins import DMSMixin


class CMISClientTests(DMSMixin, TestCase):
    def test_checkout(self):
        """
        Assert that checking out a document locks it and returns the PWC ID
        """
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        cmis_doc = self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)

        checkout_id, checkout_by = self.cmis_client.checkout(document)
        cmis_doc = self.cmis_client._get_cmis_doc(document)
        pwc = cmis_doc.getPrivateWorkingCopy()
        self.assertEqual(
            checkout_id,
            pwc.properties['cmis:versionSeriesCheckedOutId']
        )
        self.assertEqual(checkout_by, 'admin')

    def test_checkout_checked_out_doc(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)
        cmis_doc = self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)
        cmis_doc.checkout()

        with self.assertRaises(DocumentLockedException):
            self.cmis_client.checkout(document)

    def test_cancel_checkout(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)

        cmis_doc = self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)
        checkout_id, _checkout_by = self.cmis_client.checkout(document)
        # result = self.cmis_client.cancel_checkout(document, checkout_id)
        # self.assertIsNone(result)
        cmis_doc.cancelCheckout()

        # if the doc cannot be checked out, it was not unlocked
        cmis_doc = self.cmis_client._get_cmis_doc(document)
        try:
            cmis_doc.checkout()
        except UpdateConflictException:
            self.fail("Could not lock document after checkout cancel, it is still locked")

    def test_cancel_checkout_invalid_checkout_id(self):
        self.cmis_client.creeer_zaakfolder(self.zaak_url)
        document = EnkelvoudigInformatieObjectFactory.create()
        koppeling = DRCCMISConnectionFactory(enkelvoudiginformatieobject=document)

        self.cmis_client.maak_zaakdocument(koppeling, self.zaak_url)
        _checkout_id, _checkout_by = self.cmis_client.checkout(document)

        with self.assertRaises(DocumentConflictException):
            self.cmis_client.cancel_checkout(document, '')
